# -*- coding: utf-8 -*-

from DateTime import DateTime
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from zope.i18n import translate

from rer.bandi import bandiMessageFactory as _
from urllib2 import quote
try:
    from zope.app.schema.vocabulary import IVocabularyFactory
except ImportError:
    from zope.schema.interfaces import IVocabularyFactory

from zope.component import getUtility, queryUtility

try:
    from collective.solr.interfaces import ISolrConnectionConfig
    HAS_SOLR = True
except ImportError:
    HAS_SOLR = False

try:
    from collective.solr_collection.solr import solrUniqueValuesFor
    HAS_SOLR_COLLECTION = True
except ImportError:
    HAS_SOLR_COLLECTION = False


class SearchBandiForm(BrowserView):
    def __init__(self, context, request):
        self.context = context
        self.request = request

        voc_tipologia = getUtility(IVocabularyFactory, name='rer.bandi.tipologia.vocabulary')(self.context)
        self.terms_tipologia = list(voc_tipologia)
        voc_destinatari = getUtility(IVocabularyFactory, name='rer.bandi.destinatari.vocabulary')(self.context)
        self.terms_destinatari = list(voc_destinatari)
        self.solr_enabled = self.isSolrEnabled()

    def isSolrEnabled(self):
        """
        """
        if not HAS_SOLR:
            return False
        util = queryUtility(ISolrConnectionConfig)
        if util:
            return getattr(util, 'active', False)
        return False

    def getUniqueValuesForIndex(self, index):
        """
        get uniqueValuesFor a given index
        """
        if not self.solr_enabled or not HAS_SOLR_COLLECTION:
            pc = getToolByName(self, 'portal_catalog')
            return pc.uniqueValuesFor(index)
        else:
            return solrUniqueValuesFor(index, portal_type="Bando")

    def getDefaultEnte(self):
        """
        return the default ente
        """
        portal_properties = getToolByName(self, 'portal_properties')
        rer_bandi_settings = getattr(portal_properties, 'rer_bandi_settings', None)
        if rer_bandi_settings:
            return rer_bandi_settings.getProperty('default_ente', '')
        return ''


class SearchBandi(BrowserView):
    """
    A view for search bandi results
    """
    def searchBandi(self):
        """
        return a list of bandi
        """
        pc = getToolByName(self.context, 'portal_catalog')
        stato = self.request.form.get('stato_bandi', '')
        SearchableText = self.request.form.get('SearchableText', '')
        query = self.request.form.copy()
        if stato:
            now = DateTime()
            if stato == "open":
                query['getScadenza_bando'] = {'query': now, 'range': 'min'}
                query['getChiusura_procedimento_bando'] = {'query': now, 'range': 'min'}
            if stato == "inProgress":
                query['getScadenza_bando'] = {'query': now, 'range': 'max'}
                query['getChiusura_procedimento_bando'] = {'query': now, 'range': 'min'}
            if stato == "closed":
                query['getChiusura_procedimento_bando'] = {'query': now, 'range': 'max'}
        if "SearchableText" in self.request.form and not SearchableText:
            del query['SearchableText']
        return pc(**query)

    @property
    def rss_query(self):
        """
        set rss query with the right date
        """
        query = self.request.QUERY_STRING
        stato = self.request.form.get('stato_bandi', '')
        if stato:
            now = DateTime().ISO()
            if stato == "open":
                query = query + "&amp;getScadenza_bando.query:record:date=%s&getScadenza_bando.range:record=min" % quote(now)
                query = query + "&amp;getChiusura_procedimento_bando.query:record:date=%s&getChiusura_procedimento_bando.range:record=min" % quote(now)
            if stato == "inProgress":
                query = query + "&amp;getScadenza_bando.query:record:date=%s&getScadenza_bando.range:record=max" % quote(now)
                query = query + "&amp;getChiusura_procedimento_bando.query:record:date=%s&getChiusura_procedimento_bando.range:record=min" % quote(now)
            if stato == "closed":
                query = query + "&amp;getChiusura_procedimento_bando.query:record:date=%s&getChiusura_procedimento_bando.range:record=max" % quote(now)
        return query

    def getBandoState(self, bando):
        """
        """
        scadenza_bando = bando.getScadenza_bando
        chiusura_procedimento_bando = bando.getChiusura_procedimento_bando
        state = ('open', translate(_(u'Open'), context=self.request))
        if scadenza_bando and scadenza_bando.isPast():
            if chiusura_procedimento_bando and chiusura_procedimento_bando.isPast():
                state = ('closed', translate(_(u'Closed'), context=self.request))
            else:
                state = ('inProgress', translate(_(u'In progress'), context=self.request))
        else:
            if chiusura_procedimento_bando and chiusura_procedimento_bando.isPast():
                state = ('closed', translate(_(u'Closed'), context=self.request))
        return state

    def isValidDeadline(self, date):
        """
        """
        if not date:
            return False
        if date.Date() == '2100/12/31':
            #a default date for bandi that don't have a defined deadline
            return False
        return True



from zope.interface import Interface
# -*- Additional Imports Here -*-


class IBandoFolderDeepening(Interface):
    """A folder that is handled in a special way by Bando."""

    # -*- schema definition goes here -*-




from zope.interface import implements
from Products.CMFCore.utils import getToolByName
from Products.Archetypes import atapi
from Products.ATContentTypes.content import folder
from Products.ATContentTypes.content import document
from Products.ATContentTypes.content import schemata

from rer.bandi import bandiMessageFactory as _

from rer.bandi.interfaces import IBando
from rer.bandi.config import PROJECTNAME

BandoSchema = folder.ATFolderSchema.copy() + document.ATDocumentSchema.copy() + atapi.Schema((

    # -*- Your Archetypes field definitions here ... -*-

    atapi.StringField(
        'tipologia_bando',
        storage=atapi.AnnotationStorage(),
        vocabulary_factory='rer.bandi.tipologia.vocabulary',
        widget=atapi.SelectionWidget(
            label=_('tipologia_bando_label', default=u"Announcement type"),
            description=_('tipologia_bando_help', default=''),
        ),
        required=True,
    ),


    atapi.LinesField(
        'destinatari',
        storage=atapi.AnnotationStorage(),
        vocabulary_factory='rer.bandi.destinatari.vocabulary',
        widget=atapi.MultiSelectionWidget(
            label=_('destinatari_label', default=u"Recipients"),
            description=_('destinatari_help', default=''),
            format='checkbox',
        ),
    ),

    atapi.LinesField(name='ente_bando',
                     default_method="getDefaultEnte",
                     widget=atapi.KeywordWidget(
                     label=_(u'ente_label',
                                default=u'Authority'),
                        description=_(u'ente_help',
                                      default=u'Select some authorities.'),
                       ),
            ),

    atapi.DateTimeField(
        'scadenza_bando',
        storage=atapi.AnnotationStorage(),
        widget=atapi.CalendarWidget(
            label=_('scadenza_bando_label', default=u"Expiration date and time"),
            description=_('scadenza_bando_help', default=u"Deadline to participate in the announcement"),
        ),
        validators=('isValidDate'),
    ),


    atapi.DateTimeField(
        'chiusura_procedimento_bando',
        storage=atapi.AnnotationStorage(),
        widget=atapi.CalendarWidget(
            label=_('chiusura_procedimento_bando_label', default=u"Closing date procedure"),
            description=_('chiusura_procedimento_bando_help', default=u''),
            show_hm=False,
        ),
        validators=('isValidDate'),
    ),


    atapi.TextField(
        'riferimenti_bando',
        storage=atapi.AnnotationStorage(),
        searchable=True,
        default_output_type='text/x-html-safe',
        widget=atapi.RichWidget(
            label=_('riferimenti_bando_label', default=u"References"),
            description=_('riferimenti_bando_help', default=u""),
        ),
    ),


))

# Set storage on fields copied from ATFolderSchema, making sure
# they work well with the python bridge properties.

BandoSchema['title'].storage = atapi.AnnotationStorage()
BandoSchema['description'].storage = atapi.AnnotationStorage()

schemata.finalizeATCTSchema(
    BandoSchema,
    folderish=True,
    moveDiscussion=False
)

BandoSchema['relatedItems'].widget.visible = {'view': 'visible', 'edit': 'visible'}


class Bando(folder.ATFolder):
    implements(IBando)

    meta_type = "Bando"
    schema = BandoSchema


    # -*- Your ATSchema to Python Property Bridges Here ... -*-

    def getDefaultEnte(self):
        """
        return a default value for Ente
        """
        portal_properties = getToolByName(self, 'portal_properties')
        rer_bandi_settings = getattr(portal_properties, 'rer_bandi_settings', None)
        if rer_bandi_settings:
            return rer_bandi_settings.getProperty('default_ente', '')
        return ''

atapi.registerType(Bando, PROJECTNAME)



# -*- coding: utf-8 -*-

from Products.ATContentTypes.interface import IATTopic, IATContentType
from plone.app.collection.interfaces import ICollection
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone.app.form.widgets.uberselectionwidget import UberSelectionWidget
from plone.app.portlets.portlets import base
from plone.app.vocabularies.catalog import SearchableTextSourceBinder
from plone.memoize.instance import memoize
from rer.bandi import bandiMessageFactory as _
from plone.portlets.interfaces import IPortletDataProvider
from rer.bandi.interfaces import IBando
from zope import schema
from zope.component import getMultiAdapter, getUtility
from zope.formlib import form
from zope.i18n import translate
from zope.interface import implements
try:
    from zope.app.schema.vocabulary import IVocabularyFactory
except ImportError:
    from zope.schema.interfaces import IVocabularyFactory


COLLECTION_TYPES_LIST = [IATTopic.__identifier__, ICollection.__identifier__]


class IBandoCollectionPortlet(IPortletDataProvider):

    """A portlet which renders the results of a collection object.
    """

    header = schema.TextLine(title=_(u"Portlet header"),
                             description=_(u"Title of the rendered portlet"),
                             required=True)

    target_collection = schema.Choice(title=_(u"Target collection"),
                                      description=_(
                                          u"Find the collection which provides the items to list"),
                                      required=True,
                                      source=SearchableTextSourceBinder(
        {
            'object_provides': COLLECTION_TYPES_LIST
        },
        default_query='path:'
    ))

    limit = schema.Int(title=_(u"Limit"),
                       description=_(u"Specify the maximum number of items to show in the portlet. "
                                     "Leave this blank to show all items."),
                       required=False)

    show_more = schema.Bool(title=_(u"Show more... link"),
                            description=_(u"If enabled, a more... link will appear in the footer of the portlet, "
                                          "linking to the underlying Collection."),
                            required=True,
                            default=True)

    show_more_text = schema.TextLine(title=_(u"Other text"),
                                     description=_(
                                         u"Alternative text to show in 'other' link."),
                                     required=True,
                                     default=u'Altro\u2026')

    show_more_path = schema.Choice(title=_(u"Alternative link to other"),
                                   description=_(
                                       u"Select a different link to 'other'."),
                                   required=False,
                                   source=SearchableTextSourceBinder({'sort_on': 'getObjPositionInParent'},
                                                                     default_query='path:'))

    show_description = schema.Bool(
        title=u'Mostra descrizione', required=True, default=False)

    show_tipologia_bando = schema.Bool(
        title=u'Mostra tipologia bando', required=True, default=False)

    show_effective = schema.Bool(
        title=u'Mostra data di pubblicazione', required=True, default=False)

    show_scadenza_bando = schema.Bool(
        title=u'Mostra data di scadenza', required=True, default=False)


class Assignment(base.Assignment):

    """
    Portlet assignment.
    This is what is actually managed through the portlets UI and associated
    with columns.
    """

    implements(IBandoCollectionPortlet)

    header = u""
    target_collection = None
    limit = None
    show_more = True

    def __init__(self, header=u"", target_collection=None, limit=None, show_more=True, show_more_text=None, show_more_path=None,
                 show_description=False, show_tipologia_bando=False, show_effective=False, show_scadenza_bando=False):
        self.header = header
        self.target_collection = target_collection
        self.limit = limit
        self.show_more = show_more
        self.show_more_text = show_more_text
        self.show_more_path = show_more_path
        self.show_description = show_description
        self.show_tipologia_bando = show_tipologia_bando
        self.show_effective = show_effective
        self.show_scadenza_bando = show_scadenza_bando

    @property
    def title(self):
        """This property is used to give the title of the portlet in the
        "manage portlets" screen. Here, we use the title that the user gave.
        """
        return self.header


class Renderer(base.Renderer):

    """Portlet renderer.

    This is registered in configure.zcml. The referenced page template is
    rendered, and the implicit variable 'view' will refer to an instance
    of this class. Other methods can be added and referenced in the template.
    """

    _template = ViewPageTemplateFile('collection.pt')

    def __init__(self, *args):
        base.Renderer.__init__(self, *args)
        self.voc_tipologia = getUtility(
            IVocabularyFactory, name='rer.bandi.tipologia.vocabulary')(self.context)

    # Cached version - needs a proper cache key
    # @ram.cache(render_cachekey)
    # def render(self):
    #     if self.available:
    #         return xhtml_compress(self._template())
    #     else:
    #         return ''

    render = _template

    @property
    def available(self):
        return len(self.results())

    def collection_url(self):
        collection = self.collection()
        if collection is None:
            return None
        else:
            return collection.absolute_url()

    def more_target_url(self):
        """
        link target to use (either the default collection or the alternate one)
        """
        if self.data.show_more_path:
            return self.portal().absolute_url() + self.data.show_more_path

        return self.collection_url()

    def results(self):
        results = []
        collection = self.collection()
        if collection is not None:
            results = collection.queryCatalog(
                object_provides=IBando.__identifier__)
            if self.data.limit and self.data.limit > 0:
                results = results[:self.data.limit]
        return results

    def isValidDeadline(self, date):
        """
        """
        if not date:
            return False
        if date.Date() == '2100/12/31':
            #a default date for bandi that don't have a defined deadline
            return False
        return True

    @memoize
    def collection(self):
        """ get the collection the portlet is pointing to"""

        collection_path = self.data.target_collection
        if not collection_path:
            return None

        if collection_path.startswith('/'):
            collection_path = collection_path[1:]

        if not collection_path:
            return None

        portal = self.portal()
        return portal.restrictedTraverse(collection_path, default=None)

    def portal(self):
        portal_state = getMultiAdapter(
            (self.context, self.request), name=u'plone_portal_state')
        return portal_state.portal()

    def getBandoState(self, bando):
        """
        return corretc bando state
        """
        scadenza_bando = bando.getScadenza_bando
        chiusura_procedimento_bando = bando.getChiusura_procedimento_bando
        state = ('open', translate(_(u'Open'), context=self.request))
        if scadenza_bando and scadenza_bando.isPast():
            if chiusura_procedimento_bando and chiusura_procedimento_bando.isPast():
                state = (
                    'closed', translate(_(u'Closed'), context=self.request))
            else:
                state = (
                    'inProgress', translate(_(u'In progress'), context=self.request))
        else:
            if chiusura_procedimento_bando and chiusura_procedimento_bando.isPast():
                state = (
                    'closed', translate(_(u'Closed'), context=self.request))
        return state

    def has_effective_date(self, bando):
        effective_date = bando.effective.Date()
        return effective_date != 'None' and effective_date != "1000/01/01"


class AddForm(base.AddForm):

    """Portlet add form.

    This is registered in configure.zcml. The form_fields variable tells
    zope.formlib which fields to display. The create() method actually
    constructs the assignment that is being added.
    """
    form_fields = form.Fields(IBandoCollectionPortlet)
    form_fields['target_collection'].custom_widget = UberSelectionWidget
    form_fields['show_more_path'].custom_widget = UberSelectionWidget

    label = _(u"Add Bandi Portlet")
    description = _(
        u"This portlet display a listing of bandi from a Collection.")

    def create(self, data):
        return Assignment(**data)


class EditForm(base.EditForm):

    """Portlet edit form.

    This is registered with configure.zcml. The form_fields variable tells
    zope.formlib which fields to display.
    """

    form_fields = form.Fields(IBandoCollectionPortlet)
    form_fields['target_collection'].custom_widget = UberSelectionWidget
    form_fields['show_more_path'].custom_widget = UberSelectionWidget

    label = _(u"Edit Bandi Portlet")
    description = _(
        u"This portlet display a listing of bandi from a Collection.")



# -*- coding: utf-8 -*-

try:
    from zope.app.schema.vocabulary import IVocabularyFactory
except ImportError:
    from zope.schema.interfaces import IVocabularyFactory
    
from zope.interface.declarations import implements
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm

from Products.CMFCore.utils import getToolByName

class VocabularyNotFoundError(Exception):
    pass


class PropertySheetVocabulary(object):
    """
    Retrieves a vocabulary by reading a multiline property in portal_properties.
    """
    implements( IVocabularyFactory )

    def __init__(self, sheet_name, property_name, default_terms):
        self.sheet_name = sheet_name
        self.property_name = property_name
        self.default_terms = default_terms


    def iter_property_terms(self, context):
        prop_sheet = getattr(getToolByName(context, 'portal_properties'), self.sheet_name, None)
        if prop_sheet is None:
            raise VocabularyNotFoundError

        prop_lines = getattr(prop_sheet, self.property_name, None)
        if prop_lines is None:
            raise VocabularyNotFoundError

        ret = []
        for line in prop_lines:
            if '|' in line:
                value, title = line.split('|', 1)
            else:
                value = title = line
            ret.append((value.strip(), title.strip()))
        return ret


    def __call__(self, context):
        try:
            terms = self.iter_property_terms(context)
        except VocabularyNotFoundError:
            terms = self.default_terms

        return SimpleVocabulary([
                    SimpleTerm(value=t[0], token=t[0], title=t[1])
                    for t in terms
            ])





# -*- coding: utf-8 -*-


from lxml.builder import E
import lxml.etree
import os

try:
    from zope.app.schema.vocabulary import IVocabularyFactory
except ImportError:
    from zope.schema.interfaces import IVocabularyFactory

from zope.interface.declarations import implements
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm


class ConfigurationError(Exception):
    pass


class XMLFileVocabulary(object):
    """
    Retrieves a vocabulary by reading an XML file on the filesystem.
    """
    implements( IVocabularyFactory )


    def __init__(self, envvar, vocabulary_name, default_terms=None):
        """
        envvar:
            environment variable containing the path of the XML file
        vocabulary_name:
            name attribute of the <vocabulary> element
            (for instance, the name of the vocabulary utility as registered in ZCML)
        """

        if envvar not in os.environ:
            raise ConfigurationError("Missing environment variable %s" % envvar)

        filename = os.environ[envvar]

        if not os.path.exists(filename):
            fout = open(filename, 'wb')
            fout.write(self.default_file_contents(vocabulary_name, default_terms))
            fout.close()
            raise ConfigurationError("Vocabulary file file %s created with default values. Please check the contents and run again." % filename)

        fin = open(filename, 'rb')

        try:
            xml = lxml.etree.parse(fin)
        except lxml.etree.XMLSyntaxError, e:
            raise ConfigurationError("Invalid XML in file %s: %s" % (fin.name, e))

        fin.close()
        self.terms = self.extract_terms(xml, vocabulary_name)


    def extract_terms(self, xml, vocabulary_name):
        terms = []
        for el_term in xml.getroot().xpath("//vocabulary[@name='%s']/term" % vocabulary_name):
            token = el_term.attrib['token']
            try:
                value = el_term.attrib['value']
            except KeyError:
                value = token
            title = el_term.text.strip()
            terms.append(SimpleTerm(token=token, value=value, title=title))
        return sorted(terms, key=lambda x: x.title)


    def __call__(self, context):
        return SimpleVocabulary(self.terms)


    def default_file_contents(self, vocabulary_name, default_terms):
        voc = E('vocabulary', name=vocabulary_name)
        for token, title in default_terms:
            voc.append(E('term', title, token=token))
        root = E('vocab-list')
        root.append(voc)

        xmlbytes = lxml.etree.tostring(root, xml_declaration=True, encoding='utf-8', pretty_print=True)

        return xmlbytes



# -*- coding: utf-8 -*-
from rer.bandi.filevocabulary import XMLFileVocabulary
from rer.bandi.psheetvocabulary import PropertySheetVocabulary


TipologiaBandoVocabularyFactory = XMLFileVocabulary(envvar='PLONE_RER_BANDI_VOCAB',
                                                    vocabulary_name='rer.bandi.tipologia.vocabulary',
                                                    default_terms=[
                                                        ('beni_servizi',
                                                         'Acquisizione beni e servizi'),
                                                        ('agevolazioni',
                                                         'Agevolazioni, finanziamenti, contributi'),
                                                        ('altro', 'Altro'),
                                                    ])


DestinatariVocabularyFactory = PropertySheetVocabulary(sheet_name='rer_bandi_settings',
                                                       property_name='destinatari_bandi',
                                                       default_terms=[
                                                           ('Cittadini',
                                                            'Cittadini'),
                                                           ('Imprese',
                                                            'Imprese'),
                                                           ('Enti locali',
                                                            'Enti locali'),
                                                           ('Associazioni',
                                                            'Associazioni'),
                                                           ('Altro', 'Altro')
                                                       ])



"""Definition of the Bando Folder Deepening content type
"""

from zope.interface import implements

from Products.Archetypes import atapi
from Products.ATContentTypes.content import folder
from Products.ATContentTypes.content import schemata
from Products.CMFPlone.interfaces.breadcrumbs import IHideFromBreadcrumbs

# -*- Message Factory Imported Here -*-

from rer.bandi.interfaces import IBandoFolderDeepening
from rer.bandi.config import PROJECTNAME

BandoFolderDeepeningSchema = folder.ATFolderSchema.copy() + atapi.Schema((

    # -*- Your Archetypes field definitions here ... -*-

))

# Set storage on fields copied from ATFolderSchema, making sure
# they work well with the python bridge properties.

BandoFolderDeepeningSchema['title'].storage = atapi.AnnotationStorage()
BandoFolderDeepeningSchema['description'].storage = atapi.AnnotationStorage()

schemata.finalizeATCTSchema(
    BandoFolderDeepeningSchema,
    folderish=True,
    moveDiscussion=False
)

BandoFolderDeepeningSchema.changeSchemataForField('excludeFromNav', 'default')
BandoFolderDeepeningSchema['relatedItems'].widget.visible = {'view': 'visible', 'edit': 'visible'}



class BandoFolderDeepening(folder.ATFolder):
    """A folder that is handled in a special way by Bando."""
    implements(IBandoFolderDeepening, IHideFromBreadcrumbs)

    meta_type = "BandoFolderDeepening"
    schema = BandoFolderDeepeningSchema

    title = atapi.ATFieldProperty('title')
    description = atapi.ATFieldProperty('description')

    # -*- Your ATSchema to Python Property Bridges Here ... -*-

atapi.registerType(BandoFolderDeepening, PROJECTNAME)



# -*- coding: utf-8 -*-

from Products.CMFCore.utils import getToolByName
import logging
logger = logging.getLogger('rer.bandi')


def import_various(context):
    if context.readDataFile('rer.bandi_various.txt') is None:
        return

    portal = context.getSite()
    addKeyToCatalog(portal)
    addPropertySheet(portal)


def addKeyToCatalog(portal):
    setup_tool = portal.portal_setup
    setup_tool.runImportStepFromProfile('profile-rer.bandi:default', 'catalog')
    catalog = getToolByName(portal, 'portal_catalog')
    indexes = catalog.indexes()

    wanted = [
            ('getChiusura_procedimento_bando', 'DateIndex', {'indexed_attrs': 'getChiusura_procedimento_bando', }),
            ('getDestinatariBando', 'KeywordIndex', {'indexed_attrs': 'getDestinatariBando', }),
            ('getScadenza_bando', 'DateIndex', {'indexed_attrs': 'getScadenza_bando', }),
            ('getTipologia_bando', 'FieldIndex', {'indexed_attrs': 'getTipologia_bando', }),
            ('getEnte_bando', 'KeywordIndex', {'indexed_attrs': 'getEnte_bando', }),
            ]

    indexables = []
    for idx in wanted:
        if idx[0] in indexes:
            logger.info("Found the '%s' index in the catalog, nothing changed.\n" % idx[0])
        else:
            catalog.addIndex(name=idx[0], type=idx[1], extra=idx[2])
            logger.info("Added '%s' (%s) to the catalog.\n" % (idx[0], idx[1]))
            indexables.append(idx[0])
    if len(indexables) > 0:
        logger.info('Indexing new indexes %s.' % ', '.join(indexables))
        catalog.manage_reindexIndex(ids=indexables)


def addPropertySheet(portal):
    DEST = ('Cittadini|Cittadini',
          'Imprese|Imprese',
          'Enti locali|Enti locali',
          'Associazioni|Associazioni',
          'Altro|Altro')
    portal_properties = getToolByName(portal, 'portal_properties')
    rer_bandi_settings = getattr(portal_properties, 'rer_bandi_settings', None)
    if not rer_bandi_settings:
        portal_properties.addPropertySheet(id='rer_bandi_settings', title='RER Bandi settings')
        logger.info("Added RER Bandi settings property-sheet")
        rer_bandi_settings = getattr(portal_properties, 'rer_bandi_settings', None)
    if not rer_bandi_settings.hasProperty('destinatari_bandi'):
        rer_bandi_settings.manage_addProperty(id='destinatari_bandi', value=DEST, type='lines')
    if not rer_bandi_settings.hasProperty('default_ente'):
        rer_bandi_settings.manage_addProperty(id='default_ente', value="", type='string')



# -*- coding: utf-8 -*-
"""
This module contains the tool of rer.bandi
"""
import os
from setuptools import setup, find_packages

version = '2.2.2'

tests_require = ['zope.testing', 'Products.PloneTestCase']

setup(name='rer.bandi',
      version=version,
      description="A product for announcements management",
      long_description=open("README.rst").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      # Get more strings from
      # http://pypi.python.org/pypi?:action=list_classifiers
      classifiers=[
        "Programming Language :: Python",
        "Framework :: Plone",
        "Framework :: Plone :: 3.3",
        "Framework :: Plone :: 4.0",
        "Framework :: Plone :: 4.2",
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        ],
      keywords='rer bandi announcements',
      author='RedTurtle Technology',
      author_email='sviluppoplone@redturtle.it',
      url='http://plone.org/products/rer.bandi',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['rer', ],
      include_package_data=True,
      zip_safe=False,
      install_requires=['setuptools',
                        'lxml',
                        ],
      tests_require=tests_require,
      extras_require=dict(tests=tests_require),
      test_suite='rer.bandi.tests.test_docs.test_suite',
      entry_points="""
      # -*- entry_points -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



from plone.indexer.decorator import indexer
from rer.bandi.interfaces import IBando
from DateTime import DateTime


@indexer(IBando)
def destinatari_bando(object, **kw):
    return object.getDestinatari()


@indexer(IBando)
def getChiusura_procedimento_bando(object, **kw):
    if object.getChiusura_procedimento_bando():
        return object.getChiusura_procedimento_bando()
    return DateTime("2100/12/31")


@indexer(IBando)
def getScadenza_bando(object, **kw):
    if not object.getScadenza_bando():
        return DateTime('2100/12/31')
    elif object.getScadenza_bando().Time() == '00:00:00':
        return object.getScadenza_bando() + 1
    else:
        return object.getScadenza_bando()



# -*- coding: utf-8 -*-

from Products.CMFCore.utils import getToolByName
from Products.Five import BrowserView
from plone.app.layout.icons.interfaces import IContentIcon
from rer.bandi.interfaces import IBandoFolderDeepening

try:
    from zope.app.schema.vocabulary import IVocabularyFactory
except ImportError:
    from zope.schema.interfaces import IVocabularyFactory

from zope.component import getMultiAdapter, getUtility
from zope.interface import implements, Interface

class IBandoView(Interface):
    pass


class BandoView(BrowserView):
    implements(IBandoView)

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.voc_tipologia = getUtility(IVocabularyFactory, name='rer.bandi.tipologia.vocabulary')(self.context)


    def retrieveFolderDeepening(self):
        """Retrieves all Folder Deppening objects contained in Structured Document
        """
        struct_doc = self.context
        values = []
        dfolders = struct_doc.getFolderContents(contentFilter={'object_provides': IBandoFolderDeepening.__identifier__})
        for df in dfolders:
            if not df.exclude_from_nav:
                values.append(dict(title=df.Title,
                                   description=df.Description,
                                   url=df.getURL(),
                                   path=df.getPath()
                                   ))
        return values

    def retrieveContentsOfFolderDeepening(self, path_dfolder):
        """Retrieves all objects contained in Folder Deppening
        """
        values = []
        objs = self.context.portal_catalog(
            path={'query': path_dfolder, 'depth': 1},
            sort_on='getObjPositionInParent'
        )
        pp = getToolByName(self.context, 'portal_properties')
        typesUseViewActionInListings = pp.site_properties.typesUseViewActionInListings
        # ploneview = getMultiAdapter((self.context, self.request), name="plone")
        for obj in objs:
            if not obj.getPath()== path_dfolder and not obj.exclude_from_nav:
                dictfields=dict(title=obj.Title,
                                description=obj.Description,
                                url=obj.getURL(),
                                path=obj.getPath(),
                                )
                if obj.Type=='Link':
                    dictfields['url']=obj.getRemoteUrl
                if obj.Type in typesUseViewActionInListings and not obj.Type=='File':
                    dictfields['url']=obj.getURL() + "/view"
                if obj.Type=='File':
                    dictfields['url']=obj.getURL() + "/at_download/file"
                    obj_file=obj.getObject().getFile()
                    if obj_file.meta_type=='ATBlob':
                        obj_size=obj_file.get_size()
                    else:
                         obj_size=obj_file.getSize()
                    dictfields['filesize']= self.getSizeString(obj_size)
                icon = getMultiAdapter((self.context, self.request, obj), IContentIcon)
                dictfields['icon'] = icon.html_tag()
                dictfields['type'] = obj.Type
                values.append(dictfields)

        return values

    def getSizeString(self,size):
        const = {'kB':1024,
                 'MB':1024*1024,
                 'GB':1024*1024*1024}
        order = ('GB', 'MB', 'kB')
        smaller = order[-1]
        if not size:
            return '0 %s' % smaller

        if size < const[smaller]:
            return '1 %s' % smaller
        for c in order:
            if size/const[c] > 0:
                break
        return '%.2f %s' % (float(size/float(const[c])), c)

    def getDestinatariNames(self):
        """
        Return the values of destinatari vocabulary
        """
        dest_utility = getUtility(IVocabularyFactory, 'rer.bandi.destinatari.vocabulary')
        destinatari = self.context.getDestinatari()
        if not dest_utility:
            return destinatari
        dest_values = []
        dest_vocab = dest_utility(self.context)
        for dest in destinatari:
            try:
                dest_title = dest_vocab.getTerm(dest).title
            except LookupError:
                dest_title = dest
            dest_values.append(dest_title)
        return dest_values
